#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <syslog.h>

void sigchld_handler(int s){
    while(wait(NULL) > 0);
}

int main(int argc, char *argv[]) {

    struct sockaddr_in addr;
    int sock, c_sock;
    int ch;
    struct sigaction sa;
    char *answer = malloc(1000);
    char *panswer;
    FILE *pid = NULL;
    FILE *responce = NULL; 
    int debug = 0;
    int httpfile = 0;
    int backlog = 10;
    pid_t childpid;
    int answersize = 0;

    openlog(getprogname(), LOG_CONS | LOG_NDELAY | LOG_PID, LOG_USER);

    memset(&addr,0,sizeof(addr));
    addr.sin_family=AF_INET;
    addr.sin_addr.s_addr=INADDR_ANY;
    addr.sin_port=htons(9998); /* Default port is 9998 */

    while ((ch = getopt(argc, argv, "dp:P:f:b:")) != EOF)
    switch (ch) {
    case 'd':
        debug++;
        break;
    case 'f':
	if  ((responce = fopen(optarg, "r")) == NULL) {
            err(EX_NOINPUT, "Can't read file %s", optarg);
        }
	httpfile++;
        break;
    case 'p':
        addr.sin_port = htons(atoi(optarg));
        if  (atoi(optarg) < 1 || atoi(optarg) > 65535 || addr.sin_port == 0) {
            fprintf(stderr, "Port %d, must be 0 < port < 65536\n", addr.sin_port);
            exit(EX_USAGE);
        }
        break;
    case 'P':
        if  ((pid = fopen(optarg, "w")) == NULL) {
            err(EX_CANTCREAT, "Can't create PID file %s", optarg);
        }
        break;
    case 'b':
	backlog=atoi(optarg);
	if  (atoi(optarg) < 1) {
	    fprintf(stderr, "Backlog must be > 0");
	    exit(EX_USAGE);
	}
    default:
        exit(EX_USAGE);
    }
    argc -=optind;
    argv +=optind;

    if (httpfile == 0) {
	if  ((responce = fopen("/usr/local/etc/aredd.default", "r")) == NULL) {
            err(EX_NOINPUT, "Can't read file %s", optarg);
        }
    }
    while ((ch=fgetc(responce))!= EOF){
	answer[answersize]=ch;
	answersize++;
    } 
    sock = socket(PF_INET, SOCK_STREAM,0);
    if  (sock < 0 || sock >= FD_SETSIZE) {
        err(EX_OSERR, "can't create stream socket");
    }
    if  (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
        err(EX_UNAVAILABLE, "can't bind stream socket");
    }
    if  (listen(sock,backlog) < 0) {
	err(EX_UNAVAILABLE, "can't listen stream socket");
    }

    sa.sa_handler = sigchld_handler;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_RESTART;

    if (sigaction(SIGCHLD, &sa, NULL) == -1) {
	err(EX_OSERR, "can't make a sigaction(2) call");
    }
    if  (!debug && daemon(debug, debug) < 0) {
        err(EX_OSERR, "can't daemonize");
    }
    if  (pid) {
        fprintf(pid, "%d", getpid());
        fclose(pid);
    }
    for (;;) {
	c_sock=accept(sock,NULL,NULL);
	if (c_sock < 0 || sock >=FD_SETSIZE) {
	    syslog(LOG_ERR, "can't listen stream socket");
	    continue;
	}
	childpid=fork();
	if (childpid == 0) {
	    /* Child write the answer and so doesn't listen */
	    close(sock);
	    if (debug)
		printf ("Packet sent\n");
	    write(c_sock,answer,answersize);
	    exit(0);
 	}
	else if (childpid > 0) {
	    /*Parent doesn't need c_sock*/
	    close(c_sock);
	}	    
	else {
	    /* Can't fork() - so just dying */
	    syslog(LOG_ALERT, "coundn't fork(). dying");
	    exit(-1);
	}
    } 
}
